package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import java.util.logging.Logger;
import javax.xml.datatype.DatatypeConfigurationException;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.common.Strings;
import org.json.JSONObject;
import org.springframework.stereotype.Component;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.RecordType;

/**
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2021-07-06
 * @since 2014-02-17
 */
@Component
public class RecordDelivererDC extends RecordDelivererAbstract {

  private static Logger log = Logger.getLogger(RecordDelivererDC.class.getName());

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordDelivererDC(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordDelivererInterface#getRecordById(java.lang.String)
   */
  @Override
  public GetRecordType getRecordById(String id)
      throws ParseException, DatatypeConfigurationException {

    GetRecordType getRecordType = new GetRecordType();
    RecordType record = new RecordType();
    DublinCoreBuilder dublinCoreBuilder;

    String changedId = id;
    if (id.startsWith(this.repositoryObjectURIPrefix)) {
      // FIXME Why do we need changeID? Where is it used?
      changedId = changedId.replace(this.repositoryObjectURIPrefix, "");
    }

    // Get ES response, get record for given ID.
    GetResponse esResultObject = null;
    try {
      String[] includes = this.fields;
      String[] excludes = Strings.EMPTY_ARRAY;

      esResultObject = OAIPMHUtilities.getRcordByIDFromElasticSearch(OAI_ESClient.getEsIndex(), id,
          includes, excludes);

    } catch (IOException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
      log.severe("could not fetch the result of elasticsearch");
    }

    if (esResultObject.isExists()) {
      String identifier;

      // **
      // DARIAH
      // **

      if (this.dariah == true) {
        dublinCoreBuilder = putContentIntoDCFieldListsDH(esResultObject);
        JSONObject json = new JSONObject(esResultObject.getSourceAsMap());
        identifier = OAIPMHUtilities.fieldLoader(json, this.identifierField);

        String dateOfCreation = "NO_DATE_SET!";
        if (OAIPMHUtilities.fieldLoader(json, this.dateOfObjectCreation) != null) {
          try {
            dateOfCreation = OAIPMHUtilities
                .convertDateFormat(
                    OAIPMHUtilities.fieldLoader(json, this.dateOfObjectCreation).toString())
                .toXMLFormat();
          } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
          } catch (DatatypeConfigurationException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
          }
        }

        String setSpec = OAIPMHUtilities.fieldLoader(json, this.specField);
        String setSpecValue = OAIPMHUtilities.getSetSpec(setSpec, this.specFieldPrefix, identifier);

        record.setHeader(
            OAIPMHUtilities.computeResponseHeader(dateOfCreation, identifier, setSpecValue));
        record.setMetadata(dublinCoreBuilder.getDC());
      }

      // **
      // TEXTGRID
      // **

      if (this.textgrid == true
          && esResultObject.getSourceAsMap().get(this.formatField).equals(this.formatToFilter)) {

        String workUri = DublinCoreFieldLoader
            .fillListFromTGWorkValues(esResultObject,
                new String[] {this.relationToFurtherMetadataObject})
            .get(0);
        workUri = workUri.substring(this.repositoryObjectURIPrefix.length());
        dublinCoreBuilder =
            putContentIntoDCFieldListsTG(esResultObject, furtherDCElements(workUri));
        identifier = esResultObject.getSourceAsMap().get(this.identifierField).toString();
        String dateOfCreation =
            esResultObject.getSourceAsMap().get(this.dateOfObjectCreation).toString();

        String setSpec = DublinCoreFieldLoader
            .fillListFromTGWorkValues(esResultObject, TGConstants.RELATIONS_LIST).get(0);
        String setSpecValue = OAIPMHUtilities.getSetSpec(setSpec, this.specFieldPrefix, identifier);

        record.setHeader(OAIPMHUtilities.computeResponseHeader(
            OAIPMHUtilities.convertDateFormat(dateOfCreation).toString(), identifier,
            setSpecValue));
        record.setMetadata(dublinCoreBuilder.getDC());
      }

      getRecordType.setRecord(record);

    } else {
      return null;
    }

    return getRecordType;
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * @param id
   * @return
   */
  private GetResponse furtherDCElements(String id) {

    if (!id.endsWith(".0")) {
      id = id.concat(".0");
    }

    GetResponse responseWorkValues = null;
    try {
      String[] includes = this.workFields;
      String[] excludes = Strings.EMPTY_ARRAY;

      responseWorkValues = OAIPMHUtilities.getRcordByIDFromElasticSearch(OAI_ESClient.getEsIndex(),
          id, includes, excludes);

    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    return responseWorkValues;
  }

  /**
   * In case of TextGrid data from two objects are necessary. The edition object and the linked work
   * object. In result for building the DC-Object both objects from the ES-Index are needed
   * 
   * @param responseWorkValues contains the edition object
   * @param relatedWorkObject contains the work object related to edition object
   * @throws DatatypeConfigurationException
   * @throws ParseException
   */
  private DublinCoreBuilder putContentIntoDCFieldListsTG(GetResponse responseWorkValues,
      GetResponse relatedWorkObject) throws ParseException, DatatypeConfigurationException {

    DublinCoreBuilder result = new DublinCoreBuilder();

    // Set DublinCore lists with content from elastic search results due to configuration.
    result.setContributor(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.contributorList));
    result.setCoverage(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.coverageList));
    result.setCreator(
        DublinCoreFieldLoader.fillListFromTGWorkValues(relatedWorkObject, this.creatorList));
    result
        .setDate(DublinCoreFieldLoader.fillListFromTGWorkValues(relatedWorkObject, this.dateList));
    result.setDescription(
        DublinCoreFieldLoader.fillListFromTGWorkValues(relatedWorkObject, this.descriptionList));
    result.setFormat(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.formatList));
    result.setIdentifier(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.identifierList));
    result.setLanguage(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.languageList));
    result.setPublisher(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.publisherList));
    result.setRelation(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.relationList));
    result.setRights(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.rightsList));
    result.setSource(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.sourceList));
    result.setSubject(
        DublinCoreFieldLoader.fillListFromTGWorkValues(relatedWorkObject, this.subjectList));
    result.setTitle(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.titleList));
    result
        .setType(DublinCoreFieldLoader.fillListFromTGWorkValues(relatedWorkObject, this.typeList));

    return result;
  }

  /**
   * This function with just one object as parameter is for DARIAH
   * 
   * @param responseWorkValues
   * @throws DatatypeConfigurationException
   * @throws ParseException
   */
  private DublinCoreBuilder putContentIntoDCFieldListsDH(GetResponse responseWorkValues)
      throws ParseException, DatatypeConfigurationException {

    DublinCoreBuilder result = new DublinCoreBuilder();

    // Set DublinCore lists with content from elastic search results due to configuration.
    result.setContributor(OAIPMHUtilities
        .fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()), this.contributorList));
    result.setCoverage(
        OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
            this.coverageList));
    result
        .setCreator(OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
            this.creatorList));
    result.setDate(OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
        this.dateList));
    result.setDescription(
        OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
            this.descriptionList));
    result
        .setFormat(OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
            this.formatList));
    result.setIdentifier(
        OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
            this.identifierList));
    result.setLanguage(
        OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
            this.languageList));
    result.setPublisher(
        OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
            this.publisherList));
    result.setRelation(
        OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
            this.relationList));
    result
        .setRights(OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
            this.rightsList));
    result
        .setSource(OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
            this.sourceList));
    result.setSubject(OAIPMHUtilities
        .fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()), this.subjectList));
    result.setTitle(OAIPMHUtilities
        .fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()), this.titleList));
    result.setType(OAIPMHUtilities.fieldLoader(new JSONObject(responseWorkValues.getSourceAsMap()),
        this.typeList));

    return result;
  }

}
